@@ -189,7 +189,7 @@ static char *extradata2psets(AVCodecContext *c)
     }
 
     psets = av_mallocz(MAX_PSET_SIZE);
-    if (psets == NULL) {
+    if (!psets) {
         av_log(c, AV_LOG_ERROR, "Cannot allocate memory for the parameter sets.\n");
         av_free(orig_extradata);
         return NULL;
@@ -216,8 +216,8 @@ static char *extradata2psets(AVCodecContext *c)
             sps = r;
             sps_end = r1;
         }
-        if (av_base64_encode(p, MAX_PSET_SIZE - (p - psets), r, r1 - r) == NULL) {
-            av_log(c, AV_LOG_ERROR, "Cannot Base64-encode %td %td!\n", MAX_PSET_SIZE - (p - psets), r1 - r);
+        if (!av_base64_encode(p, MAX_PSET_SIZE - (p - psets), r, r1 - r)) {
+            av_log(c, AV_LOG_ERROR, "Cannot Base64-encode %"PTRDIFF_SPECIFIER" %"PTRDIFF_SPECIFIER"!\n", MAX_PSET_SIZE - (p - psets), r1 - r);
             av_free(psets);
 
             return NULL;
@@ -250,7 +250,7 @@ static char *extradata2config(AVCodecContext *c)
         return NULL;
     }
     config = av_malloc(10 + c->extradata_size * 2);
-    if (config == NULL) {
+    if (!config) {
         av_log(c, AV_LOG_ERROR, "Cannot allocate memory for the config info.\n");
         return NULL;
     }
@@ -457,7 +457,7 @@ static char *sdp_write_media_attributes(char *buff, int size, AVCodecContext *c,
                     av_log(c, AV_LOG_ERROR, "AAC with no global headers is currently not supported.\n");
                     return NULL;
                 }
-                if (config == NULL) {
+                if (!config) {
                     return NULL;
                 }
                 av_strlcatf(buff, size, "a=rtpmap:%d MPEG4-GENERIC/%d/%d\r\n"
